@@ -7,7 +7,7 @@
 #' @param ... ignored
 #' @param partial logical; allow for partial projection, if not all points of a geometry can be projected (corresponds to setting environment variable \code{OGR_ENABLE_PARTIAL_REPROJECTION} to \code{TRUE})
 #' @param check logical; perform a sanity check on resulting polygons?
-#' @param use_gdal logical; if \code{FALSE}, arguments \code{check} and \code{partial} get ignored, and projection is directly carried out by \code{lwgeom_transform} which ignores the GDAL api. This allows proj.4 parameters such as \code{+over} to act (https://github.com/r-spatial/sf/issues/511) and certain projections such as \code{wintri} which do not have an inverse, to work (https://github.com/r-spatial/sf/issues/509). This requires that sf is built by linking to \code{liblwgeom}.
+#' @param use_gdal logical; this parameter is deprecated. Use \link[lwgeom]{st_transform_proj} instead
 #' @details Transforms coordinates of object to new projection. Features that cannot be transformed are returned as empty geometries.
 #' @examples
 #' p1 = st_point(c(7,52))
@@ -63,11 +63,9 @@ st_transform.sfc = function(x, crs, ..., partial = TRUE, check = FALSE, use_gdal
 	if (missing(crs))
 		stop("argument crs cannot be missing")
 
-	if (! use_gdal) {
-		if (inherits(crs, "crs"))
-			crs = crs$proj4string
-		return(st_sfc(CPL_lwgeom_transform(x, c(st_crs(x)$proj4string, crs))))
-	}
+	if (! use_gdal)
+		.Deprecated("lwgeom::st_transform_proj", "lwgeom",
+			'install with devtools::install_github("r-spatial/lwgeom")')
 
 	crs = make_crs(crs)
 
@@ -97,7 +95,7 @@ st_transform.sfc = function(x, crs, ..., partial = TRUE, check = FALSE, use_gdal
 #' @export
 #' @examples
 #' nc = st_read(system.file("shape/nc.shp", package="sf"))
-#' st_area(nc[1,]) # area, using geosphere::areaPolygon
+#' st_area(nc[1,]) # area from long/lat
 #' st_area(st_transform(nc[1,], 32119)) # NC state plane, m
 #' st_area(st_transform(nc[1,], 2264)) # NC state plane, US foot
 #' library(units)
@@ -137,22 +135,55 @@ st_proj_info = function(type = "proj") {
     data.frame(res)
 }
 
+#' @name st_transform
+#' @export
+st_wrap_dateline = function(x, options, quiet) UseMethod("st_wrap_dateline")
+
 #' @name st_transform
 #' @param options character; should have "WRAPDATELINE=YES" to function; another parameter that is used is "DATELINEOFFSET=10" (where 10 is the default value)
 #' @param quiet logical; print options after they have been parsed?
 #' @export
 #' @examples
 #' st_wrap_dateline(st_sfc(st_linestring(rbind(c(-179,0),c(179,0))), crs = 4326))
-#' @details For a discussion of using \code{options}, see \url{https://github.com/r-spatial/sf/issues/280}
-st_wrap_dateline = function(x, options = "WRAPDATELINE=YES", quiet = TRUE) {
+#' library(maps)
+#' wrld <- st_as_sf(maps::map("world", fill = TRUE, plot = FALSE))
+#' wrld_wrap <- st_wrap_dateline(wrld, options = c("WRAPDATELINE=YES", "DATELINEOFFSET=180"),
+#'    quiet = TRUE)
+#' wrld_moll <- st_transform(wrld_wrap, "+proj=moll")
+#' plot(st_geometry(wrld_moll), col = "transparent")
+#' @details For a discussion of using \code{options}, see \url{https://github.com/r-spatial/sf/issues/280} and \url{https://github.com/r-spatial/sf/issues/541}
+st_wrap_dateline.sfc = function(x, options = "WRAPDATELINE=YES", quiet = TRUE) {
 	stopifnot(st_is_longlat(x))
 	stopifnot(is.character(options))
 	stopifnot(is.logical(quiet) && length(quiet) == 1)
 	st_sfc(CPL_wrap_dateline(x, options, quiet), crs = st_crs(x))
 }
 
+#' @name st_transform
+#' @export
+st_wrap_dateline.sf = function(x, options = "WRAPDATELINE=YES", quiet = TRUE) {
+	st_set_geometry(x, st_sfc(CPL_wrap_dateline(st_geometry(x), options, quiet), crs = st_crs(x)))
+}
+
+#' @name st_transform
+#' @export
+st_wrap_dateline.sfg = function(x, options = "WRAPDATELINE=YES", quiet = TRUE) {
+	st_sfc(CPL_wrap_dateline(st_geometry(x), options, quiet), crs = st_crs(x))[[1]]
+}
+
 st_to_s2 = function(x) {
-	# geocentric, spherical:
-	st_transform(x, 
+	# to geocentric, spherical, unit sphere:
+	st_transform(x,
 		st_crs("+proj=geocent +a=1 +b=1 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +wktext +no_defs"))
 }
+
+#' directly transform a set of coordinates
+#'
+#' directly transform a set of coordinates
+#' @param from character; proj4string of pts
+#' @param to character; target coordinate reference system
+#' @param pts two-column numeric matrix, or object that can be coerced into a matrix
+#' @export
+sf_project = function(from, to, pts) {
+	CPL_proj_direct(as.character(c(from[1], to[1])), as.matrix(pts))
+}
