@@ -1,11 +1,17 @@
 #define GEOS_USE_ONLY_R_API // prevents using non-thread-safe GEOSxx functions without _r extension.
 #include <geos_c.h>
 
-#if GEOS_VERSION_MAJOR == 3 && GEOS_VERSION_MINOR >= 5
+#if GEOS_VERSION_MAJOR == 3 
+# if GEOS_VERSION_MINOR >= 5
 #  define HAVE350
+# endif
+# if GEOS_VERSION_MINOR >= 7
+#  define HAVE370
+# endif
 #else
 # if GEOS_VERSION_MAJOR > 3
 #  define HAVE350
+#  define HAVE370
 # endif
 #endif
 
@@ -13,6 +19,8 @@
 
 #include "wkb.h"
 
+typedef int (* dist_fn)(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry *, double *);
+typedef int (* dist_parfn)(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry *, double, double *);
 typedef char (* log_fn)(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry *);
 typedef char (* log_prfn)(GEOSContextHandle_t, const GEOSPreparedGeometry *, 
 	const GEOSGeometry *);
@@ -33,7 +41,7 @@ static void __errorHandler(const char *fmt, ...) { // #nocov start
 	p = buf + strlen(buf) - 1;
 	if(strlen(buf) > 0 && *p == '\n') *p = '\0';
 
-	Rcpp::Function error("stop");
+	Rcpp::Function error(".stop_geos");
 	error(buf);
 
 	return; // #nocov end
@@ -85,13 +93,19 @@ void CPL_geos_finish(GEOSContextHandle_t ctxt) {
 
 std::vector<GEOSGeom> geometries_from_sfc(GEOSContextHandle_t hGEOSCtxt, Rcpp::List sfc, int *dim = NULL) {
 
-	double precision = sfc.attr("precision");
+	Rcpp::List sfc_cls = get_dim_sfc(sfc);
+	Rcpp::CharacterVector cls = sfc_cls["_cls"];
+	if (dim != NULL) {
+		Rcpp::IntegerVector sfc_dim = sfc_cls["_dim"];
+		if (sfc_dim.size() == 0)
+			Rcpp::stop("sfc_dim size 0: should not happen"); // #nocov
+		*dim = sfc_dim[0];
+	}
 
-	Rcpp::CharacterVector cls = get_dim_sfc(sfc, dim);
 	if (cls[0] == "XYM" || cls[0] == "XYZM")
 		Rcpp::stop("GEOS does not support XYM or XYZM geometries; use st_zm() to drop M\n"); // #nocov
 
-	Rcpp::List wkblst = CPL_write_wkb(sfc, true, native_endian(), cls, precision);
+	Rcpp::List wkblst = CPL_write_wkb(sfc, true);
 	std::vector<GEOSGeom> g(sfc.size());
 	GEOSWKBReader *wkb_reader = GEOSWKBReader_create_r(hGEOSCtxt);
 	for (int i = 0; i < sfc.size(); i++) {
@@ -117,7 +131,7 @@ Rcpp::List sfc_from_geometry(GEOSContextHandle_t hGEOSCtxt, std::vector<GEOSGeom
 		GEOSGeom_destroy_r(hGEOSCtxt, geom[i]);
 	}
 	GEOSWKBWriter_destroy_r(hGEOSCtxt, wkb_writer);
-	return CPL_read_wkb(out, true, false, native_endian());
+	return CPL_read_wkb(out, true, false);
 }
 
 Rcpp::NumericVector get_dim(double dim0, double dim1) {
@@ -226,21 +240,56 @@ Rcpp::List CPL_geos_binop(Rcpp::List sfc0, Rcpp::List sfc1, std::string op, doub
 		}
 		out.attr("dim") = get_dim(sfc0.length(), sfc1.length());
 		ret_list = Rcpp::List::create(out);
-	} else if (op == "distance") { // return double matrix:
+	} else if (op == "distance" || op == "Hausdorff" || op == "Frechet") { // return double matrix:
+		// dist_fn, dist_parfn
 		Rcpp::NumericMatrix out(sfc0.length(), sfc1.length());
-		for (size_t i = 0; i < gmv0.size(); i++) {
-			for (size_t j = 0; j < gmv1.size(); j++) {
-				double dist = -1.0;
-				if (GEOSDistance_r(hGEOSCtxt, gmv0[i], gmv1[j], &dist) == 0)
-					Rcpp::stop("GEOS error in GEOSDistance_r"); // #nocov
-				out(i, j) = dist;
+		if (par <= 0.0) {
+			dist_fn dist_function;
+			if (op == "distance")
+				dist_function = GEOSDistance_r;
+			else if (op == "Hausdorff")
+				dist_function = GEOSHausdorffDistance_r;
+#ifdef HAVE370
+			else if (op == "Frechet")
+				dist_function = GEOSFrechetDistance_r;
+#endif
+			else
+				Rcpp::stop("distance function not supported"); // #nocov
+			
+			for (size_t i = 0; i < gmv0.size(); i++) {
+				for (size_t j = 0; j < gmv1.size(); j++) {
+					double dist = -1.0;
+					if (dist_function(hGEOSCtxt, gmv0[i], gmv1[j], &dist) == 0)
+						Rcpp::stop("GEOS error in GEOS_xx_Distance_r"); // #nocov
+					out(i, j) = dist;
+				}
+				R_CheckUserInterrupt();
+			}
+		} else {
+			dist_parfn dist_function = NULL;
+			if (op == "Hausdorff")
+				dist_function = GEOSHausdorffDistanceDensify_r;
+#ifdef HAVE370
+			else if (op == "Frechet")
+				dist_function = GEOSFrechetDistanceDensify_r;
+#endif
+			else
+				Rcpp::stop("distance function not supported"); // #nocov
+
+			for (size_t i = 0; i < gmv0.size(); i++) {
+				for (size_t j = 0; j < gmv1.size(); j++) {
+					double dist = -1.0;
+					if (dist_function(hGEOSCtxt, gmv0[i], gmv1[j], par, &dist) == 0)
+						Rcpp::stop("GEOS error in GEOS_xx_Distance_r"); // #nocov
+					out(i, j) = dist;
+				}
+				R_CheckUserInterrupt();
 			}
-			R_CheckUserInterrupt();
 		}
 		ret_list = Rcpp::List::create(out);
 	} else if (op == "is_within_distance") { // return sparse matrix:
-		if (! sparse)
-			Rcpp::stop("for a dense matrix, use st_distance(x,y) <= dist");
+		if (! sparse) // taken care of at R level, use st_distance(x,y) <= z
+			Rcpp::stop("is_within_distance: shouldn't happen"); // #nocov
 		Rcpp::List sparsemat(sfc0.length());
 		for (size_t i = 0; i < gmv0.size(); i++) {
 			std::vector<size_t> sel;
@@ -433,6 +482,35 @@ Rcpp::LogicalVector CPL_geos_is_simple(Rcpp::List sfc) {
 	return out;
 }
 
+// [[Rcpp::export]]
+Rcpp::LogicalVector CPL_geos_is_empty(Rcpp::List sfc) { 
+	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
+	Rcpp::LogicalVector out(sfc.length());
+	std::vector<GEOSGeom> g = geometries_from_sfc(hGEOSCtxt, sfc, NULL);
+	for (size_t i = 0; i < g.size(); i++) {
+		out[i] = chk_(GEOSisEmpty_r(hGEOSCtxt, g[i]));
+		GEOSGeom_destroy_r(hGEOSCtxt, g[i]);
+	}
+	CPL_geos_finish(hGEOSCtxt);
+	return out;
+}
+
+// [[Rcpp::export]]
+Rcpp::List CPL_geos_normalize(Rcpp::List sfc) { // #nocov start
+	int dim = 2;
+	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
+	std::vector<GEOSGeom> gmv = geometries_from_sfc(hGEOSCtxt, sfc, &dim);
+	for (int i = 0; i < sfc.size(); i++) {
+		if (GEOSNormalize_r(hGEOSCtxt, gmv[i]) == -1)
+			Rcpp::stop("normalize: GEOS exception");
+	}
+	Rcpp::List out(sfc_from_geometry(hGEOSCtxt, gmv, dim)); // destroys gmv
+	CPL_geos_finish(hGEOSCtxt);
+	out.attr("precision") = sfc.attr("precision");
+	out.attr("crs") = sfc.attr("crs");
+	return out;
+} // #nocov end
+
 // [[Rcpp::export]]
 Rcpp::List CPL_geos_union(Rcpp::List sfc, bool by_feature = false) { 
 	int dim = 2;
@@ -457,6 +535,34 @@ Rcpp::List CPL_geos_union(Rcpp::List sfc, bool by_feature = false) {
 	return out;
 }
 
+// [[Rcpp::export]]
+Rcpp::List CPL_geos_snap(Rcpp::List sfc0, Rcpp::List sfc1, Rcpp::NumericVector tolerance) { 
+	int dim = 2;
+	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
+	std::vector<GEOSGeom> gmv0 = geometries_from_sfc(hGEOSCtxt, sfc0, &dim);
+	std::vector<GEOSGeom> gmv1 = geometries_from_sfc(hGEOSCtxt, sfc1, &dim);
+	GEOSGeom gc;
+	if (gmv1.size() > 1)
+		gc = GEOSGeom_createCollection_r(hGEOSCtxt, GEOS_GEOMETRYCOLLECTION, 
+			gmv1.data(), gmv1.size());
+	else
+		gc = gmv1[0];
+
+	std::vector<GEOSGeom> gmv_out(sfc0.size());
+	for (int i = 0; i < sfc0.size(); i++) {
+		gmv_out[i] = GEOSSnap_r(hGEOSCtxt, gmv0[i], gc, tolerance[i]);
+		if (gmv_out[i] == NULL)
+			Rcpp::stop("snap: GEOS exception"); // #nocov
+		GEOSGeom_destroy_r(hGEOSCtxt, gmv0[i]);
+	}
+	GEOSGeom_destroy_r(hGEOSCtxt, gc);
+	Rcpp::List out(sfc_from_geometry(hGEOSCtxt, gmv_out, dim)); // destroys gmv_out
+	CPL_geos_finish(hGEOSCtxt);
+	out.attr("precision") = sfc0.attr("precision");
+	out.attr("crs") = sfc0.attr("crs");
+	return out;
+}
+
 GEOSGeometry *chkNULL(GEOSGeometry *value) {
 	if (value == NULL)
 		Rcpp::stop("GEOS exception"); // #nocov
@@ -666,8 +772,9 @@ std::string CPL_geos_version(bool b = false) {
 }
 
 // [[Rcpp::export]]
-Rcpp::NumericMatrix CPL_geos_dist(Rcpp::List sfc0, Rcpp::List sfc1) {
-	Rcpp::NumericMatrix out = CPL_geos_binop(sfc0, sfc1, "distance", 0.0, "", false)[0];
+Rcpp::NumericMatrix CPL_geos_dist(Rcpp::List sfc0, Rcpp::List sfc1, 
+		Rcpp::CharacterVector which, double par) {
+	Rcpp::NumericMatrix out = CPL_geos_binop(sfc0, sfc1, Rcpp::as<std::string>(which), par, "", false)[0];
 	return out;
 }
 
@@ -705,3 +812,169 @@ Rcpp::List CPL_transpose_sparse_incidence(Rcpp::List m, int n) {
 	}
 	return out;
 }
+
+// [[Rcpp::export]]
+Rcpp::List CPL_nary_difference(Rcpp::List sfc) {
+	// initialize objects
+	int dim = 2;
+	bool contained = false;
+	std::vector<size_t> index;
+	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
+	std::vector<GEOSGeom> x = geometries_from_sfc(hGEOSCtxt, sfc, &dim);
+	std::vector<GEOSGeom> out;
+	// initialize trees to find overlapping areas quickly
+	for (size_t i = 0; i < x.size(); i++) {
+		// if i'th geometry in x is empty then skip it
+		if (! GEOSisEmpty_r(hGEOSCtxt, x[i])) {
+			GEOSSTRtree *tree = GEOSSTRtree_create_r(hGEOSCtxt, 10);
+			GEOSGeom geom = x[i];
+			// if out contains geometries than remove overlaps from geom
+			if (out.size() > 0) {
+				// generate tree for all items in out
+				std::vector<size_t> items(out.size());
+				for (size_t j = 0; j < out.size(); j++) {
+					items[j] = j;
+					if (! GEOSisEmpty_r(hGEOSCtxt, out[j])) {
+						GEOSSTRtree_insert_r(hGEOSCtxt, tree, out[j], &(items[j]));
+					}
+				}
+				// query which geometries in out overlap with geom
+				contained = false;
+				std::vector<size_t> tree_sel;
+				GEOSSTRtree_query_r(hGEOSCtxt, tree, geom, cb, &tree_sel);
+				// iterate over items in query and erase overlapping areas in geom
+				for (size_t j = 0; j < tree_sel.size(); j++) {
+					// test if the items are fully contained
+					contained = chk_(GEOSContains_r(hGEOSCtxt, out[tree_sel[j]], geom));
+					if (contained)
+						break;
+					// test if the items overlap with geom
+					if (chk_(GEOSOverlaps_r(hGEOSCtxt, geom, out[tree_sel[j]]))) {
+						// if they do then erase overlapping parts from geom
+						GEOSGeom g = GEOSDifference_r(hGEOSCtxt, geom, out[tree_sel[j]]);
+						if (g == NULL)
+							Rcpp::stop("GEOS exception"); // #nocov
+						GEOSGeom_destroy_r(hGEOSCtxt, geom); // discard
+						geom = g;
+						// ensure that geom is valid
+					}
+				}
+			}
+			// add geom to out if not empty
+			if (!contained) {
+				index.push_back(i + 1);
+				out.push_back(geom); // keep
+			} else
+				GEOSGeom_destroy_r(hGEOSCtxt, geom); // discard
+			// destroy tree
+			GEOSSTRtree_destroy_r(hGEOSCtxt, tree);
+			// check for user interrupt
+			R_CheckUserInterrupt();
+		}
+	}
+	// prepare output
+	Rcpp::List ret(sfc_from_geometry(hGEOSCtxt, out, dim)); // destroys out
+	ret.attr("crs") = sfc.attr("crs");
+	Rcpp::IntegerVector out_index = Rcpp::IntegerVector(index.begin(), index.end());
+	ret.attr("idx") = out_index;
+	// cleanup
+	CPL_geos_finish(hGEOSCtxt);
+	// return result
+	return ret;
+}
+
+// [[Rcpp::export]]
+Rcpp::List CPL_nary_intersection(Rcpp::List sfc) {
+	// initialize objects
+	int dim = 2;
+	bool contained = false;
+	std::vector< std::vector<size_t> > index;
+	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
+	std::vector<GEOSGeom> x = geometries_from_sfc(hGEOSCtxt, sfc, &dim);
+	std::vector<GEOSGeom> out;
+	// initialize trees to find overlapping areas quickly
+	for (size_t i = 0; i < x.size(); i++) {
+		// if i'th geometry in x is empty then skip it
+		if (! GEOSisEmpty_r(hGEOSCtxt, x[i])) {
+			GEOSSTRtree *tree = GEOSSTRtree_create_r(hGEOSCtxt, 10);
+			GEOSGeom geom = x[i];
+			// if out contains geometries than remove overlaps from geom
+			if (out.size() > 0) {
+				// generate tree for all items in out
+				std::vector<size_t> items(out.size());
+				for (size_t j = 0; j < out.size(); j++) {
+					items[j] = j;
+					if (! GEOSisEmpty_r(hGEOSCtxt, out[j])) {
+						GEOSSTRtree_insert_r(hGEOSCtxt, tree, out[j], &(items[j]));
+					}
+				}
+				// query which geometries in out overlap with geom
+				contained = false;
+				std::vector<size_t> tree_sel;
+				GEOSSTRtree_query_r(hGEOSCtxt, tree, geom, cb, &tree_sel);
+				// iterate over items in query and erase overlapping areas in geom
+				for (size_t j = 0; j < tree_sel.size(); j++) {
+					size_t k = tree_sel[j];
+					// test if the items are fully contained
+					GEOSGeom inters = GEOSIntersection_r(hGEOSCtxt, out[k], geom);
+					if (inters == NULL)
+						Rcpp::stop("GEOS exception"); // #nocov
+					if (! chk_(GEOSisEmpty_r(hGEOSCtxt, inters))) { // i and k intersection
+						GEOSGeom g = GEOSDifference_r(hGEOSCtxt, geom, inters); // cut out inters from geom
+						if (g == NULL)
+							Rcpp::stop("GEOS exception"); // #nocov
+						GEOSGeom_destroy_r(hGEOSCtxt, geom);
+						geom = g;
+						g = GEOSDifference_r(hGEOSCtxt, out[k], inters); // cut out inters from out[k]
+						if (g == NULL)
+							Rcpp::stop("GEOS exception"); // #nocov
+						GEOSGeom_destroy_r(hGEOSCtxt, out[k]);
+						out[k] = g;
+						out.push_back(inters); // keep
+						std::vector<size_t> idx = index[k]; // k < i, and k might already be an intersection
+						idx.push_back(i + 1);
+						index.push_back(idx);
+					} else
+						GEOSGeom_destroy_r(hGEOSCtxt, inters); // discard
+				}
+			}
+			if (! chk_(GEOSisEmpty_r(hGEOSCtxt, geom))) {
+				out.push_back(geom);
+				std::vector<size_t> idx;
+				idx.push_back(i + 1);
+				index.push_back(idx);
+			} else
+				GEOSGeom_destroy_r(hGEOSCtxt, geom); // discard
+			// destroy tree
+			GEOSSTRtree_destroy_r(hGEOSCtxt, tree);
+			// check for user interrupt
+			R_CheckUserInterrupt();
+		}
+	} // for i
+	size_t j = 0;
+    for (size_t i = 0; i < out.size(); i++) {
+        if (GEOSisEmpty_r(hGEOSCtxt, out[i]))
+			GEOSGeom_destroy_r(hGEOSCtxt, out[i]); // discard
+        else {
+			out[j] = out[i];
+			index[j] = index[i];
+			std::sort(index[j].begin(), index[j].end());
+            j++;
+		}
+    }
+	out.resize(j);
+	index.resize(j);
+	// prepare output
+	Rcpp::List ret(sfc_from_geometry(hGEOSCtxt, out, dim)); // destroys out
+	ret.attr("crs") = sfc.attr("crs");
+	Rcpp::List index_list(index.size());
+	for (size_t i = 0; i < index.size(); i++) {
+		Rcpp::IntegerVector out_index = Rcpp::IntegerVector(index[i].begin(), index[i].end());
+		index_list[i] = out_index;
+	}
+	ret.attr("idx") = index_list;
+	// cleanup
+	CPL_geos_finish(hGEOSCtxt);
+	// return result
+	return ret;
+}
