@@ -62,18 +62,20 @@ test_that("geos ops give warnings and errors on longlat", {
 	expect_silent(st_area(x))
 	expect_silent(st_length(l))
 
-	# errors:
-	expect_error(st_distance(x, y))
+	# distance on long/lat:
+	if (utils::packageVersion("lwgeom") <= "0.1-0")
+		expect_error(st_distance(x, y))
+	else
+		expect_silent(st_distance(x, y))
 })
 
 test_that("st_area() works on GEOMETRY in longlat (#131)", {
   single <- list(rbind(c(0,0), c(1,0), c(1, 1), c(0,1), c(0,0))) %>% st_polygon()
   multi <- list(single + 2, single + 4) %>% st_multipolygon()
   
-  w <- st_sfc(single, multi)
+  w <- st_sfc(single + 0.1, multi)
   expect_equal(st_area(w), 1:2)
-  expect_equal(st_area(st_set_crs(w, 4326)) %>% as.numeric(), 
-               c(12308778361, 24570125261))
+  expect_silent(st_area(st_set_crs(w, 4326))) # outcome might depend on backend used: lwgeom if proj.4 < 490, else proj.4
 })
 
 
@@ -115,7 +117,7 @@ test_that("geom operations work on sfg or sfc or sf", {
   
   expect_that(st_line_merge(lnc), is_a("sf"))
   expect_that(st_line_merge(glnc), is_a("sfc"))
-  expect_that(st_line_merge(glnc[[4]]), is_a("sfg"))
+  expect_that(st_line_merge(glnc[[3]]), is_a("sfg"))
   
   expect_silent(st_centroid(lnc))
   expect_that(st_centroid(glnc),  is_a("sfc_POINT"))
@@ -166,3 +168,70 @@ test_that("st_union works with by_feature", {
   expect_silent(z <- st_union(x[[2]], by_feature = TRUE))
   expect_silent(z <- st_union(x[[3]], by_feature = TRUE))
 })
+
+test_that("st_difference works with partially overlapping geometries", {
+	# create input testing data
+	pl1 = st_polygon(list(matrix(c(0, 0, 2, 0, 1, 1, 0 ,0), byrow = TRUE, ncol=2)))
+	pl2 = st_polygon(list(matrix(c(0, 0.5, 2, 0.5, 1, 1.5, 0, 0.5), byrow = TRUE, ncol = 2)))
+	pl3 = st_polygon(list(matrix(c(0, 1.25, 2, 1.25, 1, 2.5, 0, 1.25), byrow = TRUE, ncol = 2)))
+	in1 = st_sfc(list(pl1, pl2, pl3))
+	in2 = st_sf(order = c("A", "B", "C"), geometry = st_sfc(list(pl1, pl2, pl3), crs = 4326), agr = "constant")
+	correct_geom = st_sfc(list(
+		st_polygon(list(matrix(c(0, 2, 1, 0, 0, 0, 1, 0), ncol = 2))),
+		st_polygon(list(matrix(c(0.5, 0, 1, 2, 1.5, 1, 0.5, 0.5, 0.5, 1.5, 0.5, 0.5, 1, 0.5), ncol = 2))),
+		st_polygon(list(matrix(c(0.75, 0, 1, 2, 1.25, 1, 0.75, 1.25, 1.25, 2.5, 1.25, 1.25, 1.5, 1.25), ncol = 2)))))
+	# erase overlaps
+	out1 = st_difference(in1)
+	out2 = st_difference(in2)
+	# check that output class is correct 
+	expect_is(out1, "sfc")
+	expect_is(out2, "sf")
+	# check that output geometries are valid
+	expect_true(all(sf::st_is_valid(out1)))
+	expect_true(all(sf::st_is_valid(out2)))
+	# check that output geometries have correct attributes
+	expect_equal(attr(out1, "idx"), seq_len(3))
+	#expect_equal(attr(out2, "idx"), seq_len(3))
+	expect_equal(attr(out1, "crs"), attr(in1, "crs"))
+	expect_equal(st_crs(out2), st_crs(in2))
+	# check that output geometries are actually correct
+	expect_equal(length(out1), 3)
+	expect_equal(nrow(out2), 3)
+	expect_equal(out1[[1]][[1]], correct_geom[[1]][[1]])
+	expect_equal(out1[[2]][[1]], correct_geom[[2]][[1]])
+	expect_equal(out1[[3]][[1]], correct_geom[[3]][[1]])
+	#expect_equal(out2[[1]][[1]], correct_geom[[1]][[1]])
+	#expect_equal(out2[[2]][[1]], correct_geom[[2]][[1]])
+	#expect_equal(out2[[3]][[1]], correct_geom[[3]][[1]])
+})
+
+test_that("st_difference works with fully contained geometries", {
+	# create input testing data
+	pl1 = st_polygon(list(matrix(c(0, 0, 2, 0, 2, 2, 0, 2, 0, 0), byrow = TRUE, ncol=2)))
+	pl2 = st_polygon(list(matrix(c(0.5, 0.5, 1.5, 0.5, 1.5, 1.5, 0.5, 1.5, 0.5, 0.5), byrow = TRUE, ncol = 2)))
+	pl3 = st_polygon(list(matrix(c(5, 5, 7, 5, 7, 7, 5, 7, 5, 5), byrow = TRUE, ncol = 2)))
+	in1 = st_sfc(list(pl1, pl2, pl3))
+	in2 = st_sf(order = c("A", "B", "C"), geometry = st_sfc(list(pl1, pl2, pl3), crs = 4326), agr = "constant")
+	correct_geom = st_sfc(list(pl1, pl3))
+	# erase overlaps
+	out1 = st_difference(in1)
+	out2 = st_difference(in2)
+	# check that output class is correct
+	expect_is(out1, "sfc")
+	expect_is(out2, "sf")
+	# check that output geometries are valid
+	expect_true(all(sf::st_is_valid(out1)))
+	expect_true(all(sf::st_is_valid(out2)))
+	# check that output geometries have correct attributes
+	expect_equal(attr(out1, "idx"), c(1L, 3L))
+	#expect_equal(attr(out2, "idx"), c(1L, 3L))
+	expect_equal(attr(out1, "crs"), attr(in1, "crs"))
+	expect_equal(st_crs(out2), st_crs(in2))
+	# check that output geometries are actually correct
+	expect_equal(length(out1), 2)
+	expect_equal(length(out2), 2)
+	expect_equal(out1[[1]][[1]], correct_geom[[1]][[1]])
+	#expect_equal(out1[[2]][[1]], correct_geom[[2]][[1]])
+	#expect_equal(out2[[1]][[1]], correct_geom[[1]][[1]])
+	#expect_equal(out2[[2]][[1]], correct_geom[[2]][[1]])
+})
